import processing.serial.*;

// Screen
PImage bgImg, resetImg;
PFont font;
String line1 = "Load Type:  Axial Load";
String line2 = "Measured Strain: ";
boolean inited = false;
boolean clicked = false;
boolean getResult = false;
boolean hasPressed = false;
boolean askTare = false;
int[][] buttons = {{775, 150, 200, 125},
                   {775, 300, 200, 125}};

// Serial communication
Serial myPort; // Create object from Serial class
int portIndex = 1; // set this to the port connected to Arduino
int lowByte = 0;
int midByte = 0;
int highByte = 0;
float reading;
float vin = 4000; // bridge input voltage 
float vout; // bridge output voltage 
float GF = 2.1; // Gauge factor
float strain, prevStrain = 0;
int pos = 0;
boolean strainTared = false;
boolean tareExecuted = false;



void setup() {
  // Screen
  size(1000, 500);
  frameRate(30);
  bgImg = loadImage("Interface.png");
  resetImg = loadImage("Interface_Reset.png");
  
  // Serial communication
  println(Serial.list()); // print the list of all the ports
  println(" Connecting to -> " + Serial.list()[portIndex]);
  myPort = new Serial(this, Serial.list()[portIndex], 57600);
}

int selectButton() {
  int i;
  for (i = 0; i < 2; i++) {
    if (mouseX >= buttons[i][0] && mouseX <= buttons[i][0] + buttons[i][2]) {
      if (mouseY >= buttons[i][1] && mouseY <= buttons[i][1] + buttons[i][3]) {
        return i;
      }
    }
  }
  return -1;
}

void serialEvent(Serial anyPort) {
  if (anyPort.available() == 1) {
    if (pos == 0) {
      lowByte = anyPort.read(); // Read low 8 bits
      pos++;
    } else if (pos == 1) {
      midByte = anyPort.read(); // Read middle 8 bits
      pos++;
    } else if (pos == 2) {
      highByte = anyPort.read(); // Read high 8 bits
      //reading = lowByte + pow(16, 2) * midByte + pow(16, 4) * highByte; // Sum up HX711 reading
      reading = lowByte | (midByte << 8) | (highByte << 16); // Sum up HX711 reading
      vout = reading * 20 / 8388608;
        if(askTare == true) {
          strain = getStrain();
          prevStrain = strain;
          askTare = false;
          strainTared = true;
        }
        if (!strainTared) {
          strain = getStrain();
        }
        else {
          strain = getTaredStrain(prevStrain);
        }
      getResult = true;
      pos++;
    } else {
      anyPort.read(); // Read invalid data but without displaying
    }
  }
}

float getStrain() {
  strain = vout * 4 / ( vin * GF );
  return strain * 1000000;
}

float getTaredStrain(float prevStrain) {
  float taredStrain;
  taredStrain = getStrain() - prevStrain;
  return taredStrain;  
}

void pressButton() {
  int buttonID = selectButton();
  switch (buttonID) {
    case 0: 
      myPort.write('1'); // Write a signal to serial port
      pos = 0;
      break;
    case 1:
      if (strainTared == true) {
        strainTared = false;
        image(bgImg, 0, 0, 1000, 500);
      }
      else {
        askTare = true;
        myPort.write('1'); // Write a signal to serial port
        pos = 0;
        image(resetImg, 0, 0, 1000, 500);
      }
      break;
    case -1: 
      break;
  }
}

void printText() {
  fill(255);
  noStroke();
  rect(100, 84, 650, 150);
  
  fill(0);
  textAlign(LEFT, CENTER);
  text(line1, 100, 100);
  text(line2, 100, 200);
  text(strain, 375, 200);
  text("microstrain", 550, 200);
}

void draw() {
  if (inited == false) {
    image(bgImg, 0, 0, 1000, 500);
    font = createFont("Arial", 32);
    textFont(font);
    inited = true;
  }
  if (mousePressed == true) {
    hasPressed = true;
  }
  if (hasPressed == true && mousePressed == false) {
    pressButton();
    hasPressed = false;
  }
  if (getResult == true) {
    printText();
    getResult = false;
  }
  serialEvent( myPort );
}
